/**
 * menu.js	version 2.5.2
 * FABRE Arnaud		2004
 */

var MENU_TIMEOUT = 200;

/**
 * Register an event for an object, allowing the function to access
 * the event obj. This is an helper function in order to make menus
 * functionnal with ie.
 */
function setEvent(obj, name, action)
{
  var fn;

  if (document.all)
  {
    // ie does not understand when event is passed as parameter
    fn = 'function () {' + action + '};';
  }
  else
  {
    fn = 'function (event) {' + action + '};';
  }
  eval('fn = ' + fn);
  obj[name] = fn;
}

/**
 * Return a number that is unique within the page
 */
// Unique id for menus
var uniqId = 0;
function getUniqId()
{
  return ++uniqId;
}

function Utils()
{
}

/**
 * Format an error message
 */
Utils.prototype.doError = function (err)
{
  alert('Forum82 Menu Maker:\nAn error occured while processing a request.\n---------------------------\n' + err
	+ '\n---------------------------');
}

String.prototype.removeCssClass = function (cssClass)
{
  return this.replace(new RegExp('(\\W+|^)' + cssClass + '(\\W+|$)'), '$1');
}

String.prototype.addCssClass = function (cssClass)
{
  if (this.search(new RegExp('(\\W+|^)' + cssClass + '(\\W+|$)')) == -1)
  {
    return this + ' ' + cssClass;
  }
  else
  {
    return this;
  }
}

String.prototype.replaceCssClass = function (cssOldClass, cssNewClass)
{
  return (this.removeCssClass(cssOldClass)).addCssClass(cssNewClass);
}

/**
 * test whether a dom obj is a [tag] tag or not
 */
Utils.prototype.isHtmlTag = function (obj, tagName)
{
  return obj.tagName
  && ((obj.tagName.toUpperCase() == tagName.toUpperCase())
      || (obj.tagName.toLowerCase() == tagName.toLowerCase));
}

/**
 * return dom obj of sub-menu if it exists, null otherwise.
 * If li item contains more than one ul, return false
 */
Utils.prototype.getSubMenu = function (liObj)
{
  var childs = liObj.childNodes;
  var nbSubMenu = 0;
  var res = null;

  for (var child = 0; child < childs.length; child++)
  {
    if (this.isHtmlTag(childs[child], 'ul'))
    {
      nbSubMenu++;
      if (nbSubMenu > 1)
      {
	return false;
      }
      res = childs[child];
    }
  }
  return res;
}

/**
 * Return array of child items of an ul element
 */
Utils.prototype.getItems = function (ulObj)
{
  var childs = ulObj.childNodes;
  var items = new Array();

  for (var i = 0; i < childs.length; i++)
  {
    if (this.isHtmlTag(childs[i], 'li'))
    {
      items[items.length] = childs[i];
    }
  }
  return items;
}

/******************************************************/

function Menu()
{
  // item opening a submenu (for a menu)
  this.parentItem = new Array();
  // menus to whom items belong to (for an item)
  this.itemsMenu = new Array();
  // submenu that an item opens
  this.subMenu = new Array();
  // correspondance between id and dom obj
  this.domObj = new Array();

  // submenu closing timers
  this.subMenuTimer = new Array();

  this.openedMenu = -1;
  this.openedSubMenu = -1;

  this.HTimeout = null;
  this.HSubMenuTimeout = null;

  //////
  // Utils
  this.utils = new Utils;
}

/**
 * Gives a menu an id and update global structures
 */
Menu.prototype.registerMenu = function (menuObj, parentItem)
{
  var id = getUniqId();
  menuObj.obj_id = id;
  this.parentItem[id] = parentItem; // given an submenu id, return the parent item id
  if (parentItem != -1)
  {
    this.subMenu[parentItem] = id; // given an item id, return the submenu id
  }
  this.domObj[id] = menuObj; // save a correspondance between id and dom obj
  this.subMenuTimer[id] = null;
}

/**
 * Gives an item an id and update global structures
 */
Menu.prototype.registerItem = function (itemObj, menuId)
{
  var id = getUniqId();
  itemObj.obj_id = id;
  itemObj.menu_obj = this;
  this.domObj[id] = itemObj;
  this.subMenu[id] = -1; // no submenu yet
  this.itemsMenu[id] = menuId;
}

/**
 * return true if supposedChildId is a sub-menu of parentId
 */
Menu.prototype.isParentOf = function (parentId, supposedChildId)
{
  while (supposedChildId >= 0 && supposedChildId != parentId)
  {
    supposedChildId = this.itemsMenu[this.parentItem[supposedChildId]];
  }
  return supposedChildId == parentId;
}

/**
 * Perform a depth-first recursion on the tree
 */
Menu.prototype.init = function (idMenu)
{
  var menuBar = document.getElementById(idMenu);
  if (menuBar == null)
  {
    return false;
  }

  // Assert tag is ul
  if (!this.utils.isHtmlTag(menuBar, 'ul'))
  {
    alert('Given id does not correspond to an ul element');
    return false;
  }

  this.initSubMenu(menuBar, -1);

  // add the css class
  menuBar.className = menuBar.className.addCssClass('menu_bar');
  menuBar.className = menuBar.className.removeCssClass('normal_disp');
  this.idMenu = typeof(document.menus) == 'undefined'? 0 : document.menus.length;
  if (typeof(document.menus) == 'undefined')
  {
    document.menus = new Array();
  }
  document.menus[this.idMenu] = this;
  return true;
}

Menu.prototype.initSubMenu = function (ulObj, parentId)
{
  this.registerMenu(ulObj, parentId);

  // search items
  var li = this.utils.getItems(ulObj);
  var subMenu;
  for (var i = 0; i < li.length; i++)
  {
    this.registerItem(li[i], ulObj.obj_id);
    // continue with depth-first recursion
    subMenu = this.utils.getSubMenu(li[i]);
    if (subMenu == false)
    {
      this.utils.doError('One ul item does not correspond to a menu (multiple ul in one li detected)');
      return false;
    }
    else if (subMenu != null)
    {
      if (!this.initSubMenu(subMenu, li[i].obj_id))
      {
	return false;
      }
    }
    // do changes on the li element
    setEvent(li[i], 'onmouseover', 'itemOver(this, event);');
    setEvent(li[i], 'onmouseout', 'itemOut(this, event);');
  }

  return true;
}

Menu.prototype.closeMenu = function (menuId, upTo)
{
  if (upTo == null)
  {
    this.domObj[menuId].className = this.domObj[menuId].className.removeCssClass('opened');
  }
  else
  {
    while (menuId != -1 && menuId != upTo && (upTo != null || menuId != this.openedMenu))
    {
      this.domObj[menuId].className = this.domObj[menuId].className.removeCssClass('opened');
      if (this.parentItem[menuId] != -1)
      {
	menuId = this.itemsMenu[this.parentItem[menuId]];
      }
      else
      {
	menuId = -1;
      }
    }
  }
}
Menu.prototype.openMenu = function (menuId)
{
  // stop close timer for sub menus
  if (this.subMenuTimer[menuId] != null)
  {
    clearTimeout(this.subMenuTimer[menuId]);
    this.subMenuTimer[menuId] = null;
  }
  this.domObj[menuId].className = this.domObj[menuId].className.addCssClass('opened');
}

/***************************************************/
function itemOver(obj, event)
{
  var menuObj = obj.menu_obj;
  var itemId = obj.obj_id;
  var menuId = menuObj.itemsMenu[itemId];
  // Clear timeout
  if (menuObj.HTimeout != null)
  {
    clearTimeout(menuObj.HTimeout);
  }
  // clear submenu timeout
  if (menuObj.subMenuTimer[menuId] != null)
  {
    clearTimeout(menuObj.subMenuTimer[menuId]);
    menuObj.subMenuTimer[menuId] = null;
  }

  if (menuObj.subMenu[obj.obj_id] != -1)
  {
    if (menuObj.openedSubMenu != -1)
    {
      // First stop timer
      if (menuObj.subMenuTimer[menuObj.openedSubMenu] != null)
      {
	clearTimeout(menuObj.subMenuTimer[menuObj.openedSubMenu]);
	menuObj.subMenuTimer[menuObj.openedSubMenu] = null;
      }
      // close opened menu
      menuObj.closeMenu(menuObj.openedSubMenu, menuObj.itemsMenu[obj.obj_id]);
    }
    menuObj.openedSubMenu = menuObj.subMenu[obj.obj_id];
    menuObj.openMenu(menuObj.openedSubMenu);
  }
  else
  {
    // update opened menu and openedSubMenu
    if (menuObj.isParentOf(menuId, menuObj.openedMenu))
    {
      // close submenu with delay
      menuObj.openedSubMenu = menuObj.openedMenu;
      menuObj.subMenuTimer[menuObj.openedSubMenu] =
      setTimeout('document.menus[' + menuObj.idMenu + '].closeMenu(' + menuObj.openedSubMenu + ', ' + menuId + ')', MENU_TIMEOUT);
    }
    else
      menuObj.openedSubMenu = -1;
  }

  menuObj.openedMenu = menuId;
  obj.className = obj.className.addCssClass('hover');
  event.cancelBubble = true;
}

function itemOut(obj, event)
{
  var menuObj = obj.menu_obj;
  if (menuObj.HTimeout != null)
  {
    clearTimeout(menuObj.HTimeout);
  }
  menuObj.HTimeout = setTimeout('document.menus[' + menuObj.idMenu + '].closeMenu(' + menuObj.openedMenu
				+ ', -1)', MENU_TIMEOUT);

  // Program closing of submenu
  if (menuObj.subMenu[obj.obj_id] != -1)
  {
    if (menuObj.subMenuTimer[menuObj.subMenu[obj.obj_id]] != null)
    {
      clearTimeout(menuObj.subMenuTimer[menuObj.subMenu[obj.obj_id]]);
    }
    menuObj.subMenuTimer[menuObj.subMenu[obj.obj_id]] =
    setTimeout('document.menus[' + menuObj.idMenu + '].closeMenu(' + menuObj.subMenu[obj.obj_id]
	       + ', null)', MENU_TIMEOUT);
  }

  obj.className = obj.className.removeCssClass('hover');
  event.cancelBubble = true;
}
